package pooldemo;

import org.apache.commons.pool2.ObjectPool;
import org.apache.commons.pool2.impl.GenericObjectPool;
import org.apache.commons.pool2.impl.GenericObjectPoolConfig;
import org.apache.commons.dbcp2.*;
import org.apache.commons.dbutils.*;
import org.apache.commons.dbutils.handlers.*;

public class DBCP2App {
	private static String url = "jdbc:mysql://localhost:3306/test?useSSL=false&useUnicode=true&characterEncoding=UTF-8";
	private static String username = "root";
	private static String password = "";
	
	public static BasicDataSource getBasicDataSource() {
		BasicDataSource ds = new BasicDataSource();
		ds.setUrl(url);
		ds.setUsername(username);
		ds.setPassword(password);
		ds.setDefaultAutoCommit(true);
		return ds;
	}
	
	public static void queryBasicDataSource() throws Exception {
		BasicDataSource dataSource = getBasicDataSource();
		dataSource.start();
		QueryRunner run = new QueryRunner(dataSource);
		ResultSetHandler<User> handlerBean = new BeanHandler<User>(User.class);
		User result = run.query("select * from user where id=?", handlerBean, 1);
		System.out.println(result);
		dataSource.close();
	}
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public static PoolingDataSource<PoolableConnection> getPoolingDataSource() {
		ConnectionFactory connectionFactory = new DriverManagerConnectionFactory(url, username, password);
		PoolableConnectionFactory poolableConnectionFactory = new PoolableConnectionFactory(connectionFactory, null);
		poolableConnectionFactory.setCacheState(true);
		poolableConnectionFactory.setFastFailValidation(false);
		poolableConnectionFactory.setDefaultAutoCommit(true);
		GenericObjectPoolConfig config = new GenericObjectPoolConfig();
		config.setTestOnBorrow(true);
		ObjectPool<PoolableConnection> connectionPool = new GenericObjectPool<>(poolableConnectionFactory, config);
		PoolingDataSource<PoolableConnection> dataSource = new PoolingDataSource<>(connectionPool);
		return dataSource;
	}
	
	public static void queryPoolingDataSource() throws Exception {
		PoolingDataSource<PoolableConnection> dataSource = getPoolingDataSource();
		QueryRunner run = new QueryRunner(dataSource);
		ResultSetHandler<User> handlerBean = new BeanHandler<User>(User.class);
		User result = run.query("select * from user where id=?", handlerBean, 1);
		System.out.println(result);
		dataSource.close();
	}
	
	public static void main(String[] args) throws Exception {
		queryBasicDataSource();
		queryPoolingDataSource();
	}
}

